@@ -24,11 +24,11 @@ import os.path
 import urllib
 import functools
 
-from PyQt5.QtCore import pyqtSlot, Qt, QUrl
+from PyQt5.QtCore import pyqtSlot, Qt, QUrl, QObject
 from PyQt5.QtWebEngineWidgets import QWebEngineDownloadItem
 
 from qutebrowser.browser import downloads, pdfjs
-from qutebrowser.utils import debug, usertypes, message, log, qtutils
+from qutebrowser.utils import debug, usertypes, message, log, qtutils, objreg
 
 
 class DownloadItem(downloads.AbstractDownloadItem):
@@ -39,16 +39,19 @@ class DownloadItem(downloads.AbstractDownloadItem):
         _qt_item: The wrapped item.
     """
 
-    def __init__(self, qt_item: QWebEngineDownloadItem, parent=None):
-        super().__init__(parent)
+    def __init__(self, qt_item: QWebEngineDownloadItem,
+                 manager: downloads.AbstractDownloadManager,
+                 parent: QObject = None) -> None:
+        super().__init__(manager=manager, parent=manager)
         self._qt_item = qt_item
-        qt_item.downloadProgress.connect(  # type: ignore
+        qt_item.downloadProgress.connect(  # type: ignore[attr-defined]
             self.stats.on_download_progress)
-        qt_item.stateChanged.connect(self._on_state_changed)  # type: ignore
+        qt_item.stateChanged.connect(  # type: ignore[attr-defined]
+            self._on_state_changed)
 
         # Ensure wrapped qt_item is deleted manually when the wrapper object
         # is deleted. See https://github.com/qutebrowser/qutebrowser/issues/3373
-        self.destroyed.connect(self._qt_item.deleteLater)  # type: ignore
+        self.destroyed.connect(self._qt_item.deleteLater)
 
     def _is_page_download(self):
         """Check if this item is a page (i.e. mhtml) download."""
@@ -93,7 +96,8 @@ class DownloadItem(downloads.AbstractDownloadItem):
                              "{}".format(state_name))
 
     def _do_die(self):
-        self._qt_item.downloadProgress.disconnect()  # type: ignore
+        progress_signal = self._qt_item.downloadProgress
+        progress_signal.disconnect()  # type: ignore[attr-defined]
         if self._qt_item.state() != QWebEngineDownloadItem.DownloadInterrupted:
             self._qt_item.cancel()
 
@@ -137,14 +141,15 @@ class DownloadItem(downloads.AbstractDownloadItem):
                              "state {} (not in requested state)!".format(
                                  filename, self, state_name))
 
-    def _ask_confirm_question(self, title, msg):
+    def _ask_confirm_question(self, title, msg, *, custom_yes_action=None):
+        yes_action = custom_yes_action or self._after_set_filename
         no_action = functools.partial(self.cancel, remove_data=False)
         question = usertypes.Question()
         question.title = title
         question.text = msg
         question.url = 'file://{}'.format(self._filename)
         question.mode = usertypes.PromptMode.yesno
-        question.answered_yes.connect(self._after_set_filename)
+        question.answered_yes.connect(yes_action)
         question.answered_no.connect(no_action)
         question.cancelled.connect(no_action)
         self.cancelled.connect(question.abort)
@@ -182,6 +187,26 @@ class DownloadItem(downloads.AbstractDownloadItem):
 
         self._qt_item.accept()
 
+    def _get_conflicting_download(self):
+        """Return another potential active download with the same name.
+
+        webenginedownloads.DownloadItem needs to look for downloads both in its
+        manager and in qtnetwork-download-manager as both are used
+        simultaneously.
+
+        This method can be safely removed once #2328 is fixed.
+        """
+        conflicting_download = super()._get_conflicting_download()
+        if conflicting_download:
+            return conflicting_download
+
+        qtnetwork_download_manager = objreg.get(
+            'qtnetwork-download-manager')
+        for download in qtnetwork_download_manager.downloads:
+            if self._conflicts_with(download):
+                return download
+        return None
+
 
 def _get_suggested_filename(path):
     """Convert a path we got from chromium to a suggested filename.
@@ -241,7 +266,7 @@ class DownloadManager(downloads.AbstractDownloadManager):
         suggested_filename = _get_suggested_filename(qt_item.path())
         use_pdfjs = pdfjs.should_use_pdfjs(qt_item.mimeType(), qt_item.url())
 
-        download = DownloadItem(qt_item)
+        download = DownloadItem(qt_item, manager=self)
         self._init_item(download, auto_remove=use_pdfjs,
                         suggested_filename=suggested_filename)
 
