@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2016-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2016-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -17,9 +17,6 @@
 # You should have received a copy of the GNU General Public License
 # along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
 
-# FIXME:qtwebengine remove this once the stubs are gone
-# pylint: disable=unused-argument
-
 """QtWebEngine specific part of the web element API."""
 
 import typing
@@ -29,11 +26,10 @@ from PyQt5.QtGui import QMouseEvent
 from PyQt5.QtWidgets import QApplication
 from PyQt5.QtWebEngineWidgets import QWebEngineSettings
 
-from qutebrowser.utils import log, javascript, urlutils, usertypes
+from qutebrowser.utils import log, javascript, urlutils, usertypes, utils
 from qutebrowser.browser import webelem
 
 if typing.TYPE_CHECKING:
-    # pylint: disable=unused-import,useless-suppression
     from qutebrowser.browser.webengine import webenginetab
 
 
@@ -54,6 +50,7 @@ class WebEngineElement(webelem.AbstractWebElement):
             'class_name': str,
             'rects': list,
             'attributes': dict,
+            'is_content_editable': bool,
             'caret_position': (int, type(None)),
         }  # type: typing.Dict[str, typing.Union[type, typing.Tuple[type,...]]]
         assert set(js_dict.keys()).issubset(js_dict_types.keys())
@@ -86,7 +83,7 @@ class WebEngineElement(webelem.AbstractWebElement):
     def __eq__(self, other: object) -> bool:
         if not isinstance(other, WebEngineElement):
             return NotImplemented
-        return self._id == other._id  # pylint: disable=protected-access
+        return self._id == other._id
 
     def __getitem__(self, key: str) -> str:
         attrs = self._js_dict['attributes']
@@ -97,6 +94,7 @@ class WebEngineElement(webelem.AbstractWebElement):
         self._js_call('set_attribute', key, val)
 
     def __delitem__(self, key: str) -> None:
+        utils.unused(key)
         log.stub()
 
     def __iter__(self) -> typing.Iterator[str]:
@@ -120,9 +118,9 @@ class WebEngineElement(webelem.AbstractWebElement):
         log.stub()
         return QRect()
 
-    def classes(self) -> typing.List[str]:
+    def classes(self) -> typing.Set[str]:
         """Get a list of classes assigned to this element."""
-        return self._js_dict['class_name'].split()
+        return set(self._js_dict['class_name'].split())
 
     def tag_name(self) -> str:
         """Get the tag name of this element.
@@ -137,6 +135,9 @@ class WebEngineElement(webelem.AbstractWebElement):
         """Get the full HTML representation of this element."""
         return self._js_dict['outer_xml']
 
+    def is_content_editable_prop(self) -> bool:
+        return self._js_dict['is_content_editable']
+
     def value(self) -> webelem.JsValueType:
         return self._js_dict.get('value', None)
 
@@ -172,10 +173,12 @@ class WebEngineElement(webelem.AbstractWebElement):
 
         Args:
             elem_geometry: The geometry of the element, or None.
-                           Calling QWebElement::geometry is rather expensive so
-                           we want to avoid doing it twice.
-            no_js: Fall back to the Python implementation
+                           Ignored with QtWebEngine.
+            no_js: Fall back to the Python implementation.
+                   Ignored with QtWebEngine.
         """
+        utils.unused(elem_geometry)
+        utils.unused(no_js)
         rects = self._js_dict['rects']
         for rect in rects:
             # FIXME:qtwebengine
@@ -249,9 +252,9 @@ class WebEngineElement(webelem.AbstractWebElement):
         # (it does so with a 0ms QTimer...)
         # This is also used in Qt's tests:
         # https://github.com/qt/qtwebengine/commit/5e572e88efa7ba7c2b9138ec19e606d3e345ac90
-        qapp = QApplication.instance()
-        qapp.processEvents(QEventLoop.ExcludeSocketNotifiers |
-                           QEventLoop.ExcludeUserInputEvents)
+        QApplication.processEvents(  # type: ignore[call-overload]
+            QEventLoop.ExcludeSocketNotifiers |
+            QEventLoop.ExcludeUserInputEvents)
 
         def reset_setting(_arg: typing.Any) -> None:
             """Set the JavascriptCanOpenWindows setting to its old value."""
