@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -19,12 +19,16 @@
 
 """HTTP network cache."""
 
+import typing
 import os.path
 
 from PyQt5.QtNetwork import QNetworkDiskCache
 
 from qutebrowser.config import config
-from qutebrowser.utils import utils, qtutils
+from qutebrowser.utils import utils, qtutils, standarddir
+
+
+diskcache = typing.cast('DiskCache', None)
 
 
 class DiskCache(QNetworkDiskCache):
@@ -52,3 +56,9 @@ class DiskCache(QNetworkDiskCache):
         if not qtutils.version_check('5.9', compiled=False):
             size = 0  # pragma: no cover
         self.setMaximumCacheSize(size)
+
+
+def init(parent):
+    """Initialize the global cache."""
+    global diskcache
+    diskcache = DiskCache(standarddir.cache(), parent=parent)
