@@ -244,6 +244,10 @@ class MHTMLWriter:
         return msg
 
 
+_PendingDownloadType = typing.Set[
+    typing.Tuple[QUrl, downloads.AbstractDownloadItem]]
+
+
 class _Downloader:
 
     """A class to download whole websites.
@@ -264,9 +268,7 @@ class _Downloader:
         self.target = target
         self.writer = None
         self.loaded_urls = {tab.url()}
-        pending_download_type = typing.Set[  # pylint: disable=unused-variable
-            typing.Tuple[QUrl, downloads.AbstractDownloadItem]]
-        self.pending_downloads = set()  # type: pending_download_type
+        self.pending_downloads = set()  # type: _PendingDownloadType
         self._finished_file = False
         self._used = False
 
@@ -560,7 +562,7 @@ def start_download_checked(target, tab):
         return
     # The default name is 'page title.mhtml'
     title = tab.title()
-    default_name = utils.sanitize_filename(title + '.mhtml')
+    default_name = utils.sanitize_filename(title + '.mhtml', shorten=True)
 
     # Remove characters which cannot be expressed in the file system encoding
     encoding = sys.getfilesystemencoding()
