@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -21,7 +21,7 @@
 
 import typing
 
-from PyQt5.QtCore import QRect
+from PyQt5.QtCore import QRect, Qt
 from PyQt5.QtWebKit import QWebElement, QWebSettings
 from PyQt5.QtWebKitWidgets import QWebFrame
 
@@ -30,7 +30,6 @@ from qutebrowser.utils import log, utils, javascript, usertypes
 from qutebrowser.browser import webelem
 
 if typing.TYPE_CHECKING:
-    # pylint: disable=unused-import,useless-suppression
     from qutebrowser.browser.webkit import webkittab
 
 
@@ -58,7 +57,7 @@ class WebKitElement(webelem.AbstractWebElement):
     def __eq__(self, other: object) -> bool:
         if not isinstance(other, WebKitElement):
             return NotImplemented
-        return self._elem == other._elem  # pylint: disable=protected-access
+        return self._elem == other._elem
 
     def __getitem__(self, key: str) -> str:
         self._check_vanished()
@@ -102,9 +101,9 @@ class WebKitElement(webelem.AbstractWebElement):
         self._check_vanished()
         return self._elem.geometry()
 
-    def classes(self) -> typing.List[str]:
+    def classes(self) -> typing.Set[str]:
         self._check_vanished()
-        return self._elem.classes()
+        return set(self._elem.classes())
 
     def tag_name(self) -> str:
         """Get the tag name for the current element."""
@@ -116,6 +115,12 @@ class WebKitElement(webelem.AbstractWebElement):
         self._check_vanished()
         return self._elem.toOuterXml()
 
+    def is_content_editable_prop(self) -> bool:
+        self._check_vanished()
+        val = self._elem.evaluateJavaScript('this.isContentEditable || false')
+        assert isinstance(val, bool)
+        return val
+
     def value(self) -> webelem.JsValueType:
         self._check_vanished()
         val = self._elem.evaluateJavaScript('this.value')
@@ -176,6 +181,11 @@ class WebKitElement(webelem.AbstractWebElement):
                            self._elem.parent())
         if elem is None or elem.isNull():
             return None
+
+        if typing.TYPE_CHECKING:
+            # pylint: disable=used-before-assignment
+            assert isinstance(self._tab, webkittab.WebKitTab)
+
         return WebKitElement(elem, tab=self._tab)
 
     def _rect_on_view_js(self) -> typing.Optional[QRect]:
@@ -190,7 +200,7 @@ class WebKitElement(webelem.AbstractWebElement):
             return None
 
         text = utils.compact_text(self._elem.toOuterXml(), 500)
-        log.webelem.vdebug(  # type: ignore
+        log.webelem.vdebug(  # type: ignore[attr-defined]
             "Client rectangles of element '{}': {}".format(text, rects))
 
         for i in range(int(rects.get("length", 0))):
@@ -205,13 +215,16 @@ class WebKitElement(webelem.AbstractWebElement):
                     rect["top"] *= zoom
                     width *= zoom
                     height *= zoom
-                rect = QRect(rect["left"], rect["top"], width, height)
-                frame = self._elem.webFrame()
+                rect = QRect(int(rect["left"]), int(rect["top"]),
+                             int(width), int(height))
+
+                frame = typing.cast(typing.Optional[QWebFrame], self._elem.webFrame())
                 while frame is not None:
                     # Translate to parent frames' position (scroll position
                     # is taken care of inside getClientRects)
                     rect.translate(frame.geometry().topLeft())
                     frame = frame.parentFrame()
+
                 return rect
 
         return None
@@ -223,12 +236,14 @@ class WebKitElement(webelem.AbstractWebElement):
             geometry = self._elem.geometry()
         else:
             geometry = elem_geometry
-        frame = self._elem.webFrame()
         rect = QRect(geometry)
+
+        frame = typing.cast(typing.Optional[QWebFrame], self._elem.webFrame())
         while frame is not None:
             rect.translate(frame.geometry().topLeft())
             rect.translate(frame.scrollPosition() * -1)
-            frame = frame.parentFrame()
+            frame = typing.cast(typing.Optional[QWebFrame], frame.parentFrame())
+
         return rect
 
     def rect_on_view(self, *, elem_geometry: QRect = None,
@@ -262,6 +277,20 @@ class WebKitElement(webelem.AbstractWebElement):
         # No suitable rects found via JS, try via the QWebElement API
         return self._rect_on_view_python(elem_geometry)
 
+    def _is_hidden_css(self) -> bool:
+        """Check if the given element is hidden via CSS."""
+        attr_values = {
+            attr: self._elem.styleProperty(attr, QWebElement.ComputedStyle)
+            for attr in ['visibility', 'display', 'opacity']
+        }
+        invisible = attr_values['visibility'] == 'hidden'
+        none_display = attr_values['display'] == 'none'
+        zero_opacity = attr_values['opacity'] == '0'
+
+        is_framework = ('ace_text-input' in self.classes() or
+                        'custom-control-input' in self.classes())
+        return invisible or none_display or (zero_opacity and not is_framework)
+
     def _is_visible(self, mainframe: QWebFrame) -> bool:
         """Check if the given element is visible in the given frame.
 
@@ -270,16 +299,8 @@ class WebKitElement(webelem.AbstractWebElement):
         the tab API.
         """
         self._check_vanished()
-        # CSS attributes which hide an element
-        hidden_attributes = {
-            'visibility': 'hidden',
-            'display': 'none',
-            'opacity': '0',
-        }
-        for k, v in hidden_attributes.items():
-            if (self._elem.styleProperty(k, QWebElement.ComputedStyle) == v and
-                    'ace_text-input' not in self.classes()):
-                return False
+        if self._is_hidden_css():
+            return False
 
         elem_geometry = self._elem.geometry()
         if not elem_geometry.isValid() and elem_geometry.x() == 0:
@@ -354,7 +375,8 @@ class WebKitElement(webelem.AbstractWebElement):
             log.webelem.debug("Failed to click via JS, falling back to event")
             self._click_fake_event(click_target)
 
-    def _click_fake_event(self, click_target: usertypes.ClickTarget) -> None:
+    def _click_fake_event(self, click_target: usertypes.ClickTarget,
+                          button: Qt.MouseButton = Qt.LeftButton) -> None:
         self._tab.data.override_target = click_target
         super()._click_fake_event(click_target)
 
