@@ -330,7 +330,8 @@ class CompletionView(QTreeView):
             QItemSelectionModel.Rows)
 
         # if the last item is focused, try to fetch more
-        if idx.row() == self.model().rowCount(idx.parent()) - 1:
+        next_idx = self.indexBelow(idx)
+        if not self.visualRect(next_idx).isValid():
             self.expandAll()
 
         count = self.model().count()
