@@ -68,15 +68,19 @@ class StateConfig(configparser.ConfigParser):
         else:
             self.qt_version_changed = False
 
-        for sect in ['general', 'geometry']:
+        for sect in ['general', 'geometry', 'inspector']:
             try:
                 self.add_section(sect)
             except configparser.DuplicateSectionError:
                 pass
 
-        deleted_keys = ['fooled', 'backend-warning-shown']
-        for key in deleted_keys:
-            self['general'].pop(key, None)
+        deleted_keys = [
+            ('general', 'fooled'),
+            ('general', 'backend-warning-shown'),
+            ('geometry', 'inspector'),
+        ]
+        for sect, key in deleted_keys:
+            self[sect].pop(key, None)
 
         self['general']['qt_version'] = qt_version
         self['general']['version'] = qutebrowser.__version__
@@ -161,7 +165,7 @@ class YamlConfig(QObject):
                 # Instead, create a config.py - see :help for details.
 
             """.lstrip('\n')))
-            utils.yaml_dump(data, f)  # type: ignore
+            utils.yaml_dump(data, f)
 
     def _pop_object(self,
                     yaml_data: typing.Any,
@@ -220,7 +224,7 @@ class YamlConfig(QObject):
         migrations.changed.connect(self._mark_changed)
         migrations.migrate()
 
-        self._validate(settings)
+        self._validate_names(settings)
         self._build_values(settings)
 
     def _load_settings_object(self, yaml_data: typing.Any) -> '_SettingsType':
@@ -268,7 +272,7 @@ class YamlConfig(QObject):
         if errors:
             raise configexc.ConfigFileErrors('autoconfig.yml', errors)
 
-    def _validate(self, settings: _SettingsType) -> None:
+    def _validate_names(self, settings: _SettingsType) -> None:
         """Make sure all settings exist."""
         unknown = []
         for name in settings:
@@ -319,7 +323,7 @@ class YamlMigrations(QObject):
         self._migrate_font_replacements()
 
         self._migrate_bool('tabs.favicons.show', 'always', 'never')
-        self._migrate_bool('scrolling.bar', 'always', 'when-searching')
+        self._migrate_bool('scrolling.bar', 'always', 'overlay')
         self._migrate_bool('qt.force_software_rendering',
                            'software-opengl', 'none')
         self._migrate_renamed_bool(
@@ -332,6 +336,11 @@ class YamlMigrations(QObject):
             new_name='tabs.mode_on_change',
             true_value='persist',
             false_value='normal')
+        self._migrate_renamed_bool(
+            old_name='statusbar.hide',
+            new_name='statusbar.show',
+            true_value='never',
+            false_value='always')
 
         for setting in ['tabs.title.format',
                         'tabs.title.format_pinned',
@@ -340,10 +349,21 @@ class YamlMigrations(QObject):
                                        r'(?<!{)\{title\}(?!})',
                                        r'{current_title}')
 
+        self._migrate_to_multiple('fonts.tabs',
+                                  ('fonts.tabs.selected',
+                                   'fonts.tabs.unselected'))
+
+        self._migrate_to_multiple('content.media_capture',
+                                  ('content.media.audio_capture',
+                                   'content.media.audio_video_capture',
+                                   'content.media.video_capture'))
+
         # content.headers.user_agent can't be empty to get the default anymore.
         setting = 'content.headers.user_agent'
         self._migrate_none(setting, configdata.DATA[setting].default)
 
+        self._remove_empty_patterns()
+
     def _migrate_configdata(self) -> None:
         """Migrate simple renamed/deleted options."""
         for name in list(self._settings):
@@ -394,16 +414,19 @@ class YamlMigrations(QObject):
 
     def _migrate_font_replacements(self) -> None:
         """Replace 'monospace' replacements by 'default_family'."""
-        for name in self._settings:
+        for name, values in self._settings.items():
+            if not isinstance(values, dict):
+                continue
+
             try:
                 opt = configdata.DATA[name]
             except KeyError:
                 continue
 
-            if not isinstance(opt.typ, configtypes.Font):
+            if not isinstance(opt.typ, configtypes.FontBase):
                 continue
 
-            for scope, val in self._settings[name].items():
+            for scope, val in values.items():
                 if isinstance(val, str) and val.endswith(' monospace'):
                     new_val = val.replace('monospace', 'default_family')
                     self._settings[name][scope] = new_val
@@ -415,7 +438,11 @@ class YamlMigrations(QObject):
         if name not in self._settings:
             return
 
-        for scope, val in self._settings[name].items():
+        values = self._settings[name]
+        if not isinstance(values, dict):
+            return
+
+        for scope, val in values.items():
             if isinstance(val, bool):
                 new_value = true_value if val else false_value
                 self._settings[name][scope] = new_value
@@ -441,24 +468,59 @@ class YamlMigrations(QObject):
         if name not in self._settings:
             return
 
-        for scope, val in self._settings[name].items():
+        values = self._settings[name]
+        if not isinstance(values, dict):
+            return
+
+        for scope, val in values.items():
             if val is None:
                 self._settings[name][scope] = value
                 self.changed.emit()
 
+    def _migrate_to_multiple(self, old_name: str,
+                             new_names: typing.Iterable[str]) -> None:
+        if old_name not in self._settings:
+            return
+
+        for new_name in new_names:
+            self._settings[new_name] = {}
+            for scope, val in self._settings[old_name].items():
+                self._settings[new_name][scope] = val
+
+        del self._settings[old_name]
+        self.changed.emit()
+
     def _migrate_string_value(self, name: str,
                               source: str,
                               target: str) -> None:
         if name not in self._settings:
             return
 
-        for scope, val in self._settings[name].items():
+        values = self._settings[name]
+        if not isinstance(values, dict):
+            return
+
+        for scope, val in values.items():
             if isinstance(val, str):
                 new_val = re.sub(source, target, val)
                 if new_val != val:
                     self._settings[name][scope] = new_val
                     self.changed.emit()
 
+    def _remove_empty_patterns(self) -> None:
+        """Remove *. host patterns from the config.
+
+        Those used to be valid (and could be accidentally produced by using tSH
+        on about:blank), but aren't anymore.
+        """
+        scope = '*://*./*'
+        for name, values in self._settings.items():
+            if not isinstance(values, dict):
+                continue
+            if scope in values:
+                del self._settings[name][scope]
+                self.changed.emit()
+
 
 class ConfigAPI:
 
@@ -530,14 +592,10 @@ class ConfigAPI:
         with self._handle_error('binding', key):
             seq = keyutils.KeySequence.parse(key)
             if command is None:
-                text = ("Unbinding commands with config.bind('{key}', None) "
-                        "is deprecated. Use config.unbind('{key}') instead."
-                        .format(key=key))
-                self.errors.append(configexc.ConfigErrorDesc(
-                    "While unbinding '{}'".format(key), text))
-                self._keyconfig.unbind(seq, mode=mode)
-            else:
-                self._keyconfig.bind(seq, command, mode=mode)
+                raise configexc.Error("Can't bind {key} to None (maybe you "
+                                      "want to use config.unbind('{key}') "
+                                      "instead?)".format(key=key))
+            self._keyconfig.bind(seq, command, mode=mode)
 
     def unbind(self, key: str, mode: str = 'normal') -> None:
         """Unbind a key from a command, with an optional key mode."""
@@ -548,7 +606,9 @@ class ConfigAPI:
     def source(self, filename: str) -> None:
         """Read the given config file from disk."""
         if not os.path.isabs(filename):
-            filename = str(self.configdir / filename)
+            # We don't use self.configdir here so we get the proper file when starting
+            # with a --config-py argument given.
+            filename = os.path.join(os.path.dirname(standarddir.config_py()), filename)
 
         try:
             read_config_py(filename)
@@ -607,6 +667,17 @@ class ConfigPyWriter:
     def _gen_header(self) -> typing.Iterator[str]:
         """Generate the initial header of the config."""
         yield self._line("# Autogenerated config.py")
+        yield self._line("#")
+
+        note = ("NOTE: config.py is intended for advanced users who are "
+                "comfortable with manually migrating the config file on "
+                "qutebrowser upgrades. If you prefer, you can also configure "
+                "qutebrowser using the :set/:bind/:config-* commands without "
+                "having to write a config.py file.")
+        for line in textwrap.wrap(note):
+            yield self._line("# {}".format(line))
+
+        yield self._line("#")
         yield self._line("# Documentation:")
         yield self._line("#   qute://help/configuring.html")
         yield self._line("#   qute://help/settings.html")
@@ -696,8 +767,8 @@ def read_config_py(filename: str, raising: bool = False) -> None:
     basename = os.path.basename(filename)
 
     module = types.ModuleType('config')
-    module.config = api  # type: ignore
-    module.c = container  # type: ignore
+    module.config = api  # type: ignore[attr-defined]
+    module.c = container  # type: ignore[attr-defined]
     module.__file__ = filename
 
     try:
