@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -28,6 +28,7 @@ from PyQt5.QtCore import (pyqtSignal, pyqtSlot, QObject, QProcess,
 from qutebrowser.config import config
 from qutebrowser.utils import message, log
 from qutebrowser.misc import guiprocess
+from qutebrowser.qt import sip
 
 
 class ExternalEditor(QObject):
@@ -64,10 +65,10 @@ class ExternalEditor(QObject):
     def _cleanup(self):
         """Clean up temporary files after the editor closed."""
         assert self._remove_file is not None
-
-        watched_files = self._watcher.files() if self._watcher else []
-        if watched_files:
-            failed = self._watcher.removePaths(watched_files)
+        if (self._watcher is not None and
+                not sip.isdeleted(self._watcher) and
+                self._watcher.files()):
+            failed = self._watcher.removePaths(self._watcher.files())
             if failed:
                 log.procs.error("Failed to unwatch paths: {}".format(failed))
 
@@ -75,6 +76,8 @@ class ExternalEditor(QObject):
             # Could not create initial file.
             return
 
+        assert self._proc is not None
+
         try:
             if self._proc.exit_status() != QProcess.CrashExit:
                 os.remove(self._filename)
@@ -89,6 +92,10 @@ class ExternalEditor(QObject):
 
         Callback for QProcess when the editor was closed.
         """
+        if sip.isdeleted(self):  # pragma: no cover
+            log.procs.debug("Ignoring _on_proc_closed for deleted editor")
+            return
+
         log.procs.debug("Editor closed")
         if exitstatus != QProcess.NormalExit:
             # No error/cleanup here, since we already handle this in
@@ -164,6 +171,9 @@ class ExternalEditor(QObject):
 
     def edit_file(self, filename):
         """Edit the file with the given filename."""
+        if not os.path.exists(filename):
+            with open(filename, 'w', encoding='utf-8'):
+                pass
         self._filename = filename
         self._remove_file = False
         self._start_editor()
@@ -182,11 +192,13 @@ class ExternalEditor(QObject):
         executable = editor[0]
 
         if self._watcher:
+            assert self._filename is not None
             ok = self._watcher.addPath(self._filename)
             if not ok:
                 log.procs.error("Failed to watch path: {}"
                                 .format(self._filename))
-            self._watcher.fileChanged.connect(self._on_file_changed)
+            self._watcher.fileChanged.connect(  # type: ignore[attr-defined]
+                self._on_file_changed)
 
         args = [self._sub_placeholder(arg, line, column) for arg in editor[1:]]
         log.procs.debug("Calling \"{}\" with args {}".format(executable, args))
