@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -31,18 +31,21 @@ Module attributes:
 import io
 import operator
 import contextlib
-import typing  # pylint: disable=unused-import,useless-suppression
+import typing
 
 import pkg_resources
 from PyQt5.QtCore import (qVersion, QEventLoop, QDataStream, QByteArray,
-                          QIODevice, QSaveFile, QT_VERSION_STR,
-                          PYQT_VERSION_STR, QFileDevice, QObject)
+                          QIODevice, QFileDevice, QSaveFile, QT_VERSION_STR,
+                          PYQT_VERSION_STR, QObject, QUrl)
 from PyQt5.QtGui import QColor
 from PyQt5.QtWidgets import QApplication
 try:
     from PyQt5.QtWebKit import qWebKitVersion
 except ImportError:  # pragma: no cover
-    qWebKitVersion = None  # type: ignore  # noqa: N816
+    qWebKitVersion = None  # type: ignore[assignment]  # noqa: N816
+if typing.TYPE_CHECKING:
+    from PyQt5.QtWebKit import QWebHistory
+    from PyQt5.QtWebEngineWidgets import QWebEngineHistory
 
 from qutebrowser.misc import objects
 from qutebrowser.utils import usertypes
@@ -61,23 +64,27 @@ MINVALS = {
 
 class QtOSError(OSError):
 
-    """An OSError triggered by a QFileDevice.
+    """An OSError triggered by a QIODevice.
 
     Attributes:
         qt_errno: The error attribute of the given QFileDevice, if applicable.
     """
 
-    def __init__(self, dev: QFileDevice, msg: str = None) -> None:
+    def __init__(self, dev: QIODevice, msg: str = None) -> None:
         if msg is None:
             msg = dev.errorString()
 
+        self.qt_errno = None  # type: typing.Optional[QFileDevice.FileError]
+        if isinstance(dev, QFileDevice):
+            msg = self._init_filedev(dev, msg)
+
         super().__init__(msg)
 
-        self.qt_errno = None  # type: typing.Optional[QFileDevice.FileError]
-        try:
-            self.qt_errno = dev.error()
-        except AttributeError:
-            pass
+    def _init_filedev(self, dev: QFileDevice, msg: str) -> str:
+        self.qt_errno = dev.error()
+        filename = dev.fileName()
+        msg += ": {!r}".format(filename)
+        return msg
 
 
 def version_check(version: str,
@@ -100,7 +107,7 @@ def version_check(version: str,
         # qVersion() ==/>= parsed, now check if QT_VERSION_STR ==/>= parsed.
         result = op(pkg_resources.parse_version(QT_VERSION_STR), parsed)
     if compiled and result:
-        # FInally, check PYQT_VERSION_STR as well.
+        # Finally, check PYQT_VERSION_STR as well.
         result = op(pkg_resources.parse_version(PYQT_VERSION_STR), parsed)
     return result
 
@@ -116,10 +123,11 @@ def is_new_qtwebkit() -> bool:
             pkg_resources.parse_version('538.1'))
 
 
-def is_single_process():
+def is_single_process() -> bool:
     """Check whether QtWebEngine is running in single-process mode."""
     if objects.backend == usertypes.Backend.QtWebKit:
         return False
+    assert objects.backend == usertypes.Backend.QtWebEngine, objects.backend
     args = QApplication.instance().arguments()
     return '--single-process' in args
 
@@ -150,7 +158,16 @@ def check_overflow(arg: int, ctype: str, fatal: bool = True) -> int:
         return arg
 
 
-def ensure_valid(obj: QObject) -> None:
+if typing.TYPE_CHECKING:
+    class Validatable(typing.Protocol):
+
+        """An object with an isValid() method (e.g. QUrl)."""
+
+        def isValid(self) -> bool:
+            ...
+
+
+def ensure_valid(obj: 'Validatable') -> None:
     """Ensure a Qt object with an .isValid() method is valid."""
     if not obj.isValid():
         raise QtValueError(obj)
@@ -170,7 +187,16 @@ def check_qdatastream(stream: QDataStream) -> None:
         raise OSError(status_to_str[stream.status()])
 
 
-def serialize(obj: QObject) -> QByteArray:
+_QtSerializableType = typing.Union[
+    QObject,
+    QByteArray,
+    QUrl,
+    'QWebEngineHistory',
+    'QWebHistory'
+]
+
+
+def serialize(obj: _QtSerializableType) -> QByteArray:
     """Serialize an object into a QByteArray."""
     data = QByteArray()
     stream = QDataStream(data, QIODevice.WriteOnly)
@@ -178,28 +204,34 @@ def serialize(obj: QObject) -> QByteArray:
     return data
 
 
-def deserialize(data: QByteArray, obj: QObject) -> None:
+def deserialize(data: QByteArray, obj: _QtSerializableType) -> None:
     """Deserialize an object from a QByteArray."""
     stream = QDataStream(data, QIODevice.ReadOnly)
     deserialize_stream(stream, obj)
 
 
-def serialize_stream(stream: QDataStream, obj: QObject) -> None:
+def serialize_stream(stream: QDataStream, obj: _QtSerializableType) -> None:
     """Serialize an object into a QDataStream."""
+    # pylint: disable=pointless-statement
     check_qdatastream(stream)
-    stream << obj  # pylint: disable=pointless-statement
+    stream << obj  # type: ignore[operator]
     check_qdatastream(stream)
 
 
-def deserialize_stream(stream: QDataStream, obj: QObject) -> None:
+def deserialize_stream(stream: QDataStream, obj: _QtSerializableType) -> None:
     """Deserialize a QDataStream into an object."""
+    # pylint: disable=pointless-statement
     check_qdatastream(stream)
-    stream >> obj  # pylint: disable=pointless-statement
+    stream >> obj  # type: ignore[operator]
     check_qdatastream(stream)
 
 
 @contextlib.contextmanager
-def savefile_open(filename, binary=False, encoding='utf-8'):
+def savefile_open(
+        filename: str,
+        binary: bool = False,
+        encoding: str = 'utf-8'
+) -> typing.Iterator[typing.IO]:
     """Context manager to easily use a QSaveFile."""
     f = QSaveFile(filename)
     cancelled = False
@@ -208,10 +240,12 @@ def savefile_open(filename, binary=False, encoding='utf-8'):
         if not open_ok:
             raise QtOSError(f)
 
+        dev = typing.cast(typing.BinaryIO, PyQIODevice(f))
+
         if binary:
-            new_f = PyQIODevice(f)
+            new_f = dev  # type: typing.IO
         else:
-            new_f = io.TextIOWrapper(PyQIODevice(f), encoding=encoding)
+            new_f = io.TextIOWrapper(dev, encoding=encoding)
 
         yield new_f
 
@@ -268,7 +302,7 @@ class PyQIODevice(io.BufferedIOBase):
         if not self.writable():
             raise OSError("Trying to write to unwritable file!")
 
-    def open(self, mode):
+    def open(self, mode: QIODevice.OpenMode) -> contextlib.closing:
         """Open the underlying device and ensure opening succeeded.
 
         Raises OSError if opening failed.
@@ -289,10 +323,10 @@ class PyQIODevice(io.BufferedIOBase):
         """Close the underlying device."""
         self.dev.close()
 
-    def fileno(self):
+    def fileno(self) -> int:
         raise io.UnsupportedOperation
 
-    def seek(self, offset, whence=io.SEEK_SET):
+    def seek(self, offset: int, whence: int = io.SEEK_SET) -> int:
         self._check_open()
         self._check_random()
         if whence == io.SEEK_SET:
@@ -307,7 +341,9 @@ class PyQIODevice(io.BufferedIOBase):
         if not ok:
             raise QtOSError(self.dev, msg="seek failed!")
 
-    def truncate(self, size=None):
+        return self.dev.pos()
+
+    def truncate(self, size: int = None) -> int:
         raise io.UnsupportedOperation
 
     @property
@@ -325,27 +361,33 @@ class PyQIODevice(io.BufferedIOBase):
     def readable(self) -> bool:
         return self.dev.isReadable()
 
-    def readline(self, size=-1):
+    def readline(self, size: typing.Optional[int] = -1) -> bytes:
         self._check_open()
         self._check_readable()
 
-        if size < 0:
+        if size is None or size < 0:
             qt_size = 0  # no maximum size
         elif size == 0:
-            return QByteArray()
+            return b''
         else:
             qt_size = size + 1  # Qt also counts the NUL byte
 
+        buf = None  # type: typing.Union[QByteArray, bytes, None]
         if self.dev.canReadLine():
             buf = self.dev.readLine(qt_size)
+        elif size is None or size < 0:
+            buf = self.dev.readAll()
         else:
-            if size < 0:
-                buf = self.dev.readAll()
-            else:
-                buf = self.dev.read(size)
+            buf = self.dev.read(size)
 
         if buf is None:
             raise QtOSError(self.dev)
+
+        if isinstance(buf, QByteArray):
+            # The type (bytes or QByteArray) seems to depend on what data we
+            # feed in...
+            buf = buf.data()
+
         return buf
 
     def seekable(self) -> bool:
@@ -359,23 +401,36 @@ class PyQIODevice(io.BufferedIOBase):
     def writable(self) -> bool:
         return self.dev.isWritable()
 
-    def write(self, b: bytes) -> int:
+    def write(  # type: ignore[override]
+            self,
+            data: typing.Union[bytes, bytearray]
+    ) -> int:
         self._check_open()
         self._check_writable()
-        num = self.dev.write(b)
-        if num == -1 or num < len(b):
+        num = self.dev.write(data)
+        if num == -1 or num < len(data):
             raise QtOSError(self.dev)
         return num
 
-    def read(self, size=-1):
+    def read(self, size: typing.Optional[int] = None) -> bytes:
         self._check_open()
         self._check_readable()
-        if size < 0:
+
+        buf = None  # type: typing.Union[QByteArray, bytes, None]
+        if size in [None, -1]:
             buf = self.dev.readAll()
         else:
+            assert size is not None
             buf = self.dev.read(size)
+
         if buf is None:
             raise QtOSError(self.dev)
+
+        if isinstance(buf, QByteArray):
+            # The type (bytes or QByteArray) seems to depend on what data we
+            # feed in...
+            buf = buf.data()
+
         return buf
 
 
@@ -383,9 +438,9 @@ class QtValueError(ValueError):
 
     """Exception which gets raised by ensure_valid."""
 
-    def __init__(self, obj: QObject) -> None:
+    def __init__(self, obj: 'Validatable') -> None:
         try:
-            self.reason = obj.errorString()
+            self.reason = obj.errorString()  # type: ignore[attr-defined]
         except AttributeError:
             self.reason = None
         err = "{} is not valid".format(obj)
@@ -405,7 +460,11 @@ class EventLoop(QEventLoop):
         super().__init__(parent)
         self._executing = False
 
-    def exec_(self, flags=QEventLoop.AllEvents):
+    def exec_(
+            self,
+            flags: QEventLoop.ProcessEventsFlags =
+            typing.cast(QEventLoop.ProcessEventsFlags, QEventLoop.AllEvents)
+    ) -> int:
         """Override exec_ to raise an exception when re-running."""
         if self._executing:
             raise AssertionError("Eventloop is already running!")
